package com.example.api;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @author frankzohu
 * @Description 实体类转换工具
 */
public class BeanConvertUtils {

    /**
     * 单实体转化
     */
    public static final <Target, Source> Target copyModelProperties(Source source, Class<Target> targetClass) {
        try {
            if (source == null || targetClass == null) {
                return null;
            }
            Target doInstance = targetClass.newInstance();
            BeanUtils.copyProperties(source, doInstance);
            return doInstance;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    /**
     * List集合转化
     */
    public static final <Target, Source> List<Target> copyListProperties(List<Source> sourceList, Class<Target> targetClass) {
        try {
            if (null == sourceList || sourceList.size() == 0) {
                return new ArrayList<>();
            }
            List<Target> result = new ArrayList<>();
            for (Source source : sourceList) {
                Target target = copyModelProperties(source, targetClass);
                result.add(target);
            }
            return result;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }


    /**
     * Page分页类型转化
     */
    public static final <Target, Source> IPage<Target> copyPageProperties(IPage<Source> pages, Class<Target> targetClass) {
        try {
            List<Target> records = copyListProperties(pages.getRecords(), targetClass);
            IPage<Target> pageTarget = new Page(pages.getCurrent(), pages.getSize(), pages.getTotal());
            pageTarget.setRecords(records);
            return pageTarget;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 全信息集合转id集合
     */
    public static final <Source> List<Long> getListIdProperties(List<Source> sourceList) {
        try {
            if (null == sourceList || sourceList.size() == 0) {
                return new ArrayList<>();
            }
            List<Long> result = new ArrayList<>();
            for (Source source : sourceList) {
                result.add(Long.parseLong(source.getClass().getMethod("getId").invoke(source).toString()));
            }
            return result;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

}
