package com.example.services.impl;


import com.example.utils.DateUtils;
import lombok.SneakyThrows;
import net.sf.jsqlparser.expression.DateTimeLiteralExpression;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.concurrent.CopyOnWriteArraySet;
import java.util.concurrent.Semaphore;

@ServerEndpoint(value = "/test/{sid}")
@Component
public class WebSocketServerContent {
    private String sid = "";
    ServiceImpl ServiceImpl = new ServiceImpl();
    //    RFIDController RFIDController = new RFIDController();
    //静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
    private static int onlineCount = 0;
    //concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。若要实现服务端与单一客户端通信的话，可以使用Map来存放，其中Key可以为用户标识
    private static CopyOnWriteArraySet<WebSocketServerContent> webSocketSet = new CopyOnWriteArraySet<WebSocketServerContent>();
    //与某个客户端的连接会话，需要通过它来给客户端发送数据
    private Session session;

    private static String RecMessage = "";

    /**
     * 连接建立成功调用的方法
     *
     * @param session 可选的参数。session为与某个客户端的连接会话，需要通过它来给客户端发送数据
     */
    @OnOpen
    public void onOpen(Session session, @PathParam("sid") String sid) {
        this.session = session;
        this.sid = sid;
        webSocketSet.add(this);     //加入set中
        addOnlineCount();           //在线数加1
        System.out.println("有一连接加入！" + session.getId() + "当前在线人数为" + getOnlineCount() + ",IP:" + sid);
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
        webSocketSet.remove(this);  //从set中删除
        subOnlineCount();           //在线数减1
        System.out.println("有一连接关闭！当前在线人数为" + getOnlineCount());
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message 客户端发送过来的消息
     */
    @OnMessage
    public void onMessage(String message, Session session) throws InterruptedException {
        System.out.println("手持机发来的信息是:" + message);
        ServiceImpl.DealValidPack(message);
        //RecMessage = message;
    }

    public static String getRecMessage() {
        return RecMessage;
    }

    @OnError
    public void onError(Session session, Throwable error) {
        System.out.println("未断开连接强退发生错误");
//        error.printStackTrace();
    }

    @Bean
    public void Runable() {
        Thread t1 = new Thread(new Runnable() {
            @SneakyThrows
            public void run() {
                DealMessageDoWork();
            }
        });
        t1.start();
    }

    private void DealMessageDoWork() throws InterruptedException {
        while (true) {
            if (ServiceImpl != null) {
                if (!RecMessage.isEmpty() || !RecMessage.equals("")) {
                    ServiceImpl.DealValidPack(RecMessage);
                    RecMessage = "";
                }
//                System.out.println(DateUtils.getTime() +"一开始就运行");
                Thread.sleep(1000);
            }

        }
    }

    /**
     * 服务端发送消息给客户端
     */
    public void sendMessage(String message, Session toSession) {
        try {
            System.out.println("服务端给客户端发送消息" + message);
            toSession.getBasicRemote().sendText(message);
        } catch (Exception e) {
            System.out.println("服务端发送消息给客户端失败");
            e.printStackTrace();
        }
    }

    public static synchronized boolean getSocket(String id, String message) {
        boolean isFlag = false;
        try {

            if (id.equals("") || id == null) {
                for (WebSocketServerContent i : webSocketSet) {
                    i.sendMessage(message, i.session);
                    isFlag = true;
                }
            } else {
                for (WebSocketServerContent i : webSocketSet) {
                    if (i.sid.equals(id)) {
                        i.sendMessage(message, i.session);
                        isFlag = true;
                    }
                }
            }
        } catch (Exception ex) {
            return false;
        }
        return isFlag;
    }

    public static synchronized int getOnlineCount() {
        return onlineCount;
    }

    public static synchronized void addOnlineCount() {
        WebSocketServerContent.onlineCount++;
    }

    public static synchronized void subOnlineCount() {
        WebSocketServerContent.onlineCount--;
    }
}